program cmap

  ! Original version by M. Hamabe 1999-07-23 (SPIRAL version2.6)
  ! Updated by H. Shimizu 2007-09-21 (for Fortran90)
  ! Updated by M. Hamabe  2007-10-23 (opninp)

  real, allocatable :: Ga(:,:), Gb(:,:)
  
  character image1*80, image2*80, image*80
  character yn, comments*50
  character titleA*40, titleB*40
  integer   im1, im2, nim, irc, bitpix
  
  call opninp (1,im1,0,image1,titleA,iextA,jextA,irc)
  if ( irc/=0 ) go to 999
  call opninp (2,im2,0,image2,titleB,iextB,jextB,irc)
  if ( irc/=0 ) go to 999
  
  call clargc (3, image, irc)
  if ( irc /= 0 ) then
     print '(''Enter       output   image name: '',$)'
     read *, image
     irc=0
  end if
  
  call clargr (4, sigma, irc)
  if ( irc /= 0 ) then
     sigma=3.0
     irc=0
  end if
  
  hclip=100.0
  
  allocate( Ga(iextA, jextA) )
  call MATRD(im1,Ga,iextA,jextA,irc)
  
  call GETKEYE(im1, 'UNIT-MAG', skyA, irc)
  if ( irc /= 0 ) then
     print '(''Enter unit magnitude (sky-1, if Isky=1)  ===> '',$)'
     read *, skyA
     irc=0
  end if
  call FCLOSE(im1,irc)
  if ( irc /= 0 ) go to 999
  
  allocate( Gb(iextB, jextB) )
  call MATRD(im2,Gb,iextB,jextB,irc)
  
  call GETKEYE(im2, 'UNIT-MAG', skyB, irc)
  if ( irc /= 0 ) then
     print '(''Enter unit magnitude (sky-2, if Isky=1)  ===> '',$)'
     read *, skyB
     irc=0
  end if
  
  !     call imclos(im2,irc)
  if ( irc /= 0 ) go to 999
  
  !     *** CHECK SIZES OF MATRICES ***
  IF( iextA /= iextB.OR.jextA /= jextB ) THEN
     print *, '### Matrix sizes are not same !!! ###'
     GO TO 9000
  END IF
  iext=iextB
  jext=jextB
  
  !     *** DISPLAY STATISTICS ON TERMINAL ***
  IS=1
  ie=iext/6
  JS=1
  JE=jext/6
  FACT=3.0
  NCYCLE=2
  call STATRJ(Ga,iext,jext,IS,IE,JS,JE,FACT,NCYCLE, &
       &      GMAX,GMIN,GMEAN,gsd,NREJ,KRET)
  gsdA=gsd
  
  call STATRJ(Gb,iext,jext,IS,IE,JS,JE,FACT,NCYCLE, &
       &      GMAX,GMIN,GMEAN,gsd,NREJ,KRET)
  gsdB=gsd
  
  crtA=gsdA*sigma
  crtB=gsdB*sigma
  call imdivc(Ga,Gb,iext,jext,crtA,crtB,hclip)
  deallocate(Ga)
  
  print '(''OK? '',$)'
  read '(a)', yn
  if ( yn == 'n' ) then
     print *, 'color image will not be saved.'
     stop
  end if
  bitpix=-32
  call IMGCOPY(im2,nim,bitpix,image,irc)
  if ( irc /= 0 ) goto 999
  
  call MATWT (nim,Gb,iext,jext,irc)
  deallocate(Gb)
  
  call adhstry(nim,'COLOR',5,'RATIO OF TWO IMAGES',irc)
  comments='BLUE IMAGE: '//image1
  call adhstry(nim,'COLOR',0,comments,irc)
  comments='RED  IMAGE: '//image2
  call adhstry(nim,'COLOR',0,comments,irc)
  comments='(most part of the header is for red image)'
  call adhstry(nim,'COLOR',0,comments,irc)
  write(comments,'(''LOWEST-VALUE ='',f5.2,'' (SIGMA)'')') sigma
  call adhstry(nim,'COLOR',0,comments,irc)
  sky=skyA-skyB
  print *, 'UNIT-MAG (color) =',sky
  call FTPKYF(nim,'UNIT-MAG',sky,4,'UNIT OF BRIGHTNESS (mag/arcsec**2)',irc)
  if ( irc /= 0 ) go to 999
  call FCLOSE(nim,irc)
  if (irc /= 0) go to 999
  
9000 STOP
  
999 call PUTEMSG(irc)
  stop
  
END program cmap
!----------------------------------------------------------------------
SUBROUTINE imdivc(Ga,Gb,iext,jext,crtA,crtB,hclip)
  
  DIMENSION Ga(iext,jext),Gb(iext,jext)
  
  do j=1,jext
     do i=1, iext
        if ( Ga(i,j) > crtA .and. Gb(i,j) > crtB ) then
           dum=Ga(i,j)/Gb(i,j)
        else
           dum=-999.0
        end if
        if ( dum > hclip ) dum=hclip
        Gb(I,J)=dum
     end do
  end do
  
  RETURN
END SUBROUTINE imdivc
