program eqprof1 
  parameter (NPT=51, npt1=52)
  
  ! Original version by M. Hamabe 2002-01-28 (SPIRAL version2.6)
  ! Updated by H. Shimizu 2007-10-16 (for Fortran90)
  ! Checked by M. Hamabe  2007-10-22

  character image*80, tmpfile*40, outfile*80, dsrc*1
  integer   im, irc, hard
  character title*40, yn, ichr, scl, model, device*10
  dimension SLOG(NPT),SINT(NPT)
  dimension Re(npt1),Nmesh(npt1),SP(npt1)
  dimension U(NPT),V(NPT),YCRS(2)
  logical   lexist
  data PI/3.141593/
  
  lout=21
  
  call opninp (1,im,0,image,title,iext,jext,irc)
  if ( irc/=0 ) then
     call PUTEMSG(irc)
     stop
  end if
  leng_im=len_trim(image)
  outfile=image(:leng_im)//'.eqprf'
  inquire (file=outfile, exist=lexist)
  if ( lexist ) then
     print '(''ERROR: Cannot open file   '',a)', outfile(1:20)
     stop
  end if
  
  call clargc (2, tmpfile, irc)
  if ( irc/=0 ) then
     print '(''Enter name of temporary file ===> '',$)'
     read '(a)', tmpfile
     irc=0
  end if
  open (lout, file=tmpfile, access='SEQUENTIAL')
  write(lout,'(a80)') image
  
  call clargc (3, dsrc, irc)
  if ( irc/=0 .or. dsrc/='p' ) then
     irc=0
     dsrc='c'
  end if
  
  call clargc (4, device, irc)
  if ( irc/=0 ) then
     device='/xterm'
     irc=0
  end if
  if ( device(1:3)=='tek' ) then
     device='/xterm'
  else if ( device(1:5)=='xserv' ) then
     device='/xserve'
  else if ( device(1:4)=='xwin' ) then
     device='/xwindow'
  else
     device='inq'
  end if
  
  call clargc (5, model, irc)
  if ( irc/=0 .or. model/='d' ) then
     model='e'
     irc=0
  end if
  
  call GETKEYE(im,'CDELT1  ',psize,irc)
  if ( irc/=0 ) call GETKEYE(im,'SECPPIX ',psize,irc)
  psize=abs(psize)
  !     if psize is very small then unit will be assumed to be 'degree'.
  if ( psize<10/3600.0 ) psize=psize*3600.0
  if ( psize<0.001 .or. psize>10.0 ) irc=1
  if ( irc/=0 ) then
     print '(''Enter pixel size in unit of arcsec/pixel      ===> '',$)'
     read *, PSIZE
  end if
  print '(''     Pixel size ='',f6.3,''(arcsec)'')', psize
  
  !     *** SET CENTER ***
  print *
  print '(''Enter pixel coordinate of galaxy center (I,J) ===> '',$)'
  read  *, CX,CY
  IC=INT(CX+0.5)
  JC=INT(CY+0.5)
  
  if ( dsrc=='p' ) then
     call GETKEYE(im, 'UNIT-MAG', sky, irc)
     if ( irc/=0 ) then
        print '(''Enter sky surface brightness ===> '',$)' 
        read *, sky
     end if
  else
     call GETKEYE(im,'EXPTIME', expt, irc)
     if ( irc/=0 ) then
        print '(''Enter exposure time  ===> '',$)'
        read *, expt
        irc=0
     end if
     print '(''exposure = '',f7.2,''(sec)'')', expt
     call GETKEYE(im,'PHOTZERO', pzero, irc)
     if ( irc/=0 ) then
        print *, '*** Photemetric ZERO-point (PZP) ***'
        print *, '  PZP is the magnitude corresponding to 1 ADU/sec'
        print '(''Enter PZP  ===> '',$)'
        read *, pzero
     end if
     sky = pzero+2.5*log10(psize*psize*expt)
     print '(''pixel value = 1  <===> '',f7.3,'' mag arcsec**(-2)'')', sky
  end if

  !**** read PARAMETERS FOR EQUIVALENT PROFILE *****

  print *
  print *, 'ENTER Limiting Radius (pixel unit)'
  print *, '      Axial Ratio (b/a)'
  print *, '      Position Angle (from the top, clock','wise)'
  yn='n'
  do while( yn=='n' )
     print '(''Enter  a(limit), b/a, P.A. ? ===> '',$)'
     read *, A,AXRAT,THETA
     print *
     print '(''A, AXRAT, THETA:'',3F8.1)',  A,AXRAT,THETA
     print '(''Are parameters OK? (<y>/n) ===> '',$)'
     read '(A)', YN
  end do
  THETA=THETA+90.0
  if (THETA>360) THETA=THETA-360
  RADIAN=THETA*PI/180.0
  SIZE=PSIZE/60.0
  AREA=SIZE*SIZE

  do i=1,NPT
     SLOG(i)=4.0-0.1*(I-1)
     SINT(i)=10.0**SLOG(i)
  end do
 
  call LEVEL1(IC,JC,A,AXRAT,RADIAN,PEAK,SINT,Nmesh,SP,NN,NT,iext,jext,im)

  write(lout,'(A40,40X/ "Pix.size =",G15.7," (arcmin**2)"/ &
         &  "Center   =",2I5/"Sky mag. =",F10.4," (mag/arcsec**2)"/ &
         &  "Maj.axis =",F10.4," (meshes)"/"Axis rat.=",F10.4/ &
         &  "Pos. ang.=",F10.4," (deg.)"/"N(total) =",I10/ &
         &  "N(effect)=",I10)') title,AREA,IC,JC,SKY,A,AXRAT,THETA,NT,NN

  do i=1, npt1
     SP(i)=SP(i)*AREA
  end do
  
  call EQPPRF (PEAK,SKY,SLOG,RE,Nmesh,SP,AREA,lout)
  
  !**** PLOT EQUIVALENT PROFILE **********
  
  print *
  print *, 'ENTER parameters for calculation of mag(total)'
  print *
  if( model=='d' ) then
     write(lout,'(''de Vaucouleurs'')')
     SCL='r'
  else
     model='e'
     write(lout,'(''exponential   '')')
     SCL='l'
  end if
  
  hard=0
  if ( device=='inq' ) then
     call PGBEG(0,'?',1,1)
  else
     call PGBEG(0,device,1,1)
  end if
  U1=0.0
  UDUM=0.0
  do i=1,NPT
     UDUM=MAX(UDUM,Re(i)*60.0)
  end do
  UMAX=INT( (UDUM*1.2)/10.0+0.5 )*10
  if( scl=='r' ) then
     U2=INT(UMAX**(0.25)/0.5+1.0)*0.5
     Ndat=0
     do i=1,NPT
        if( Re(i)>0.0 ) then
           Ndat=Ndat+1
           U(Ndat)=( Re(i)*60.0 )**(0.25)
           V(Ndat)=SLOG(i)
        end if
     end do
  else
     Ndat=0
     U2=UMAX
     do i=1,NPT
        if( Re(i)>0.0 ) then
           Ndat=Ndat+1
           U(Ndat)=  Re(i)*60.0
           V(Ndat)=SLOG(i)
        end if
     end do
  end if
  V1=-2.5
  V2=5.0

2000 call PGENV(u1,u2,v1,v2,-2,-2)
  call PGBOX('BCNST',0.0,-1,'BCNSTV',0.0,-1)
  if( scl=='r' ) then
     call PGLABEL('R\\\\u1/4\\\\d (arcsec)','log Intensity',' ')
  else
     call PGLABEL('R(arcsec)','log Intensity',' ')
  end if
  call PGIDEN
  call PGPT(ndat,u,v,17)
  
  print *
  if ( hard==1 ) goto 3000
  
  print *
  print *, 'Profile will be extrapolated beyond logI(Limit)'
  print *, '        using the data logI(1) & logI(Limit)'
  print *, '             ( logI(1) > logI(Limit) )'
  print *
  yn='n'
  do while ( yn=='n' )
     print *, ' Give  logI(1) and logI(Limit) by cursor'
91   call PGCURS(xxcrs,yycrs,ichr)
     ic=ichar(ichr)
     if( ic==68 .or. ic==100 .or. ic==205 .or. ic==237 ) then
!        xcrs(1)=xxcrs
        ycrs(1)=yycrs
        goto 92
     end if
     goto 91
92   call PGCURS(xxcrs,yycrs,ichr)
     ic=ichar(ichr)
     if( ic==68 .or. ic==100 .or. ic==205  .or. ic==237 ) then
!        xcrs(2)=xxcrs
        ycrs(2)=yycrs
        goto 93
     end if
     goto 92
     
93   YDUM1=ycrs(1)
     YDUM2=ycrs(2)
     Slimit=YDUM1
     Alimit=YDUM2
     if ( YDUM1<YDUM2 ) then
        Slimit=YDUM2
        Alimit=YDUM1
     end if
     print '(''  logI(1) ='',F7.2,''   logI(Limit)='',F7.2)', Slimit, Alimit
     print '(''   OK ? ( <y>/n ) ===> '',$)'
     read '(A1)', YN
  end do

  write(lout,'(''log I(1) ='',F10.1)') Slimit
  write(lout,'(''log I(L) ='',F10.1)') Alimit
  
3000 if ( hard==0 ) then
     call PGend
     print *
     yn='n'
     if ( yn=='y' ) hard=1
     if (hard==1 ) then
        call PGBEGIN(0,'eqprof1.ps/VPS',1,1)
        call PGSCF(2)
        goto 2000
     end if
  end if
  
  if ( hard==1 ) then
     call PGend
  end if
  
end program eqprof1

!____ LEVEL1 _________________________________________________________
SUBROUTINE LEVEL1(IC,JC,A,RAT,TH,PEAK,STEP,Nmesh,SP,NN,NT,iext,jext,im)
  parameter (NPT=51, npt1=52)
  
  dimension STEP(NPT),Nmesh(npt1),SP(npt1)
  dimension d(4096),CS(5),KK(5)
  
  CS(1)=STEP(11)
  CS(2)=STEP(21)
  CS(3)=STEP(31)
  CS(4)=STEP(41)
  CS(5)=STEP(51)
  KK(1)=1
  KK(2)=11
  KK(3)=21
  KK(4)=31
  KK(5)=41
  NN=0
  NT=0
  ALPHA=RAT*RAT
  S=SIN( TH )
  C=COS( TH )
  S=S*S
  C=C*C
  AA=S+ALPHA*C
  AB=(ALPHA-1.0)*SIN( 2.0*TH )
  BB=C+ALPHA*S
  RLIM=A*A*ALPHA

  do i=1,npt1
     Nmesh(i)=0
     SP(i)=0.0
  end do

  PEAK=0.0
  do J=1,JEXT
     igroup=1
     nullval=0
     call FTGPVE(im,igroup,(j-1)*iext+1,iext,nullval,d,anyf,irc)
     do i=1,IEXT
        X=I-IC
        Y=JC-J
        R=AA*X*X+AB*X*Y+BB*Y*Y
        if ( R > RLIM ) go to 30
        NT=NT+1
        P=D(i)
        if ( P <= 0.0 ) go to 30
        if( PEAK<P ) PEAK=P
        if ( P >= STEP(1) ) then
           Nmesh(1)=Nmesh(1)+1
           SP(1)=SP(1)+P
           NN=NN+1
        else
           do JJ=1,5
              if ( P >= CS(JJ) ) then
                 JS=KK(JJ)
                 do JJJ=1,10
                    JK=JS+JJJ
                    if ( P>=STEP(JK) ) then
                       Nmesh(JK)=Nmesh(JK)+1
                       SP(JK)=SP(JK)+P
                       NN=NN+1
                       go to 30
                    end if
                 end do
              end if
           end do
           Nmesh(npt1)=Nmesh(npt1)+1
           SP(npt1)=SP(npt1)+P
           NN=NN+1
        end if
30      continue
     end do
  end do
  
  return
  
end SUBROUTINE LEVEL1
     
!____ EQPPRF _________________________________________________________
SUBROUTINE EQPPRF(PEAK,SKY,SLOG,R,Nmesh,SP,AREA,lout)
  parameter (NPT=51, npt1=52)
  dimension SLOG(NPT),Nmesh(npt1),SP(npt1)
  dimension DA(npt1),SUMP(npt1),A(npt1),NSUM(npt1),R(npt1)
  data PAI/3.141593/
  
  DA=Nmesh*AREA
  
  A(1)=DA(1)
  SUMP(1)=SP(1)
  NSUM(1)=Nmesh(1)
  R(1)=SQRT( A(1)/PAI )
  do i=1,NPT
     A(I+1)=A(i)+DA(I+1)
     SUMP(I+1)=SUMP(i)+SP(I+1)
     NSUM(I+1)=NSUM(i)+Nmesh(I+1)
     R(I+1)=SQRT( A(I+1)/PAI )
  end do
  
  write(lout,'("PEAK INT.=",G15.7/5X,"MAG.",6X,"LOG I", &
       &  5X,"EQ. RAD.",10X,"N",7X,"DELTA L",7X,"SIGMA L")') PEAK
  
  do i=1,NPT
     AMAG=-2.5*SLOG(i)+SKY
     write(lout,'(2F10.3,G15.7,I10,2G15.7)')  &
          & AMAG,SLOG(i),R(i),Nmesh(i),SP(i),SUMP(i)
  end do
  
  
  return
  
end SUBROUTINE EQPPRF
