program unitmag
  
  ! Original version by M. Hamabe 1999-07-23 (SPIRAL version2.6)
  ! Updated by H. Shimizu 2007-08-03 (for Fortran90)
  ! Updated by M. Hamabe  2007-10-23 (opninp)
  
  real, allocatable :: G(:,:) 
  character :: image*80, comments*50, outfile*80
  character :: ctlg*3, hcopy*5, scale*5, band*1, rng*5, hdev*6, pid*5
  character :: TITLE*40, LONGOCAT*80, device*10, hfile*40
  character :: SOURCE(300)*10, REJECT(300), PROFDIR*80
  integer   :: im, irc, bitpix
  real      :: AP(300),AMAG(300),DMAG(300),GROW(501)
  logical   :: there

  include 'SPIRAL.INC'
  
  !     *** SET LOGICAL UNIT NUMBER AND INITIALIZATION ***
  nlp =16
  lin =11
  
  open (lin, file=LONGOCAT,access='SEQUENTIAL')
  
  print *
  call opninp (1,im,0,image,title,iext,jext,irc) ! open image in R/W mode
  if ( irc/=0 ) go to 999
  call opniaf(image,'umag',outfile,nlp,irc)
  if ( irc /= 0 ) stop
  
  call GETKEYE(im, 'CDELT1  ',PSIZE, irc)
  if ( irc /= 0 ) call GETKEYE(im,'SECPPIX ',PSIZE, irc)
  psize=abs(psize)
  if ( psize < 10/3600.0 ) psize=psize*3600.0
  if ( psize < 0.001 .or. psize > 10.0 ) irc=1
  if ( irc /= 0 ) then
     print '(''Enter pixel size in unit of arcsec/pixel    ===> '',$)'
     read *, PSIZE
     irc=0
  end if
  print '(''     Pixel size ='',f6.3,''(arcsec)'')', PSIZE
  
  call clargr (2, CENTI, irc)
  if ( irc /= 0 .or. CENTI <= 0.0 ) then
     print '(''Enter x-coordinate of galaxy center (pixel) ===> '',$)'
     read *, CENTI
  end if
  call clargr (3, CENTJ, irc)
  if ( irc /= 0 .or. CENTJ <= 0.0 ) then
     print '(''Enter y-coordinate of galaxy center (pixel) ===> '',$)'
     read *, CENTJ
  end if
  print *
  print '(''galaxy center ('',f7.1,'','',f7.1,'')'')', centi,centj
  
  call CLARGC (4, ctlg, irc)
  if ( irc /= 0 ) ctlg='yes'
  call CLARGC (5, band, irc)
  if ( irc /= 0 ) band ='v'
  call CLARGC (6, rng, irc)
  if ( irc /= 0 ) rng ='no'
  call CLARGC (7, scale, irc)
  if ( irc /= 0 ) scale ='yes'
  call CLARGC (8, device, irc)
  if ( irc /= 0 ) device='/xterm'
  if ( device(1:3) == 'tek' ) then
     device='/xterm'
  else if ( device(1:4) == 'xser' ) then
     device='/xserve'
  else if ( device(1:4) == 'xwin' ) then
     device='/xwindow'
  else
     device='inq'
  end if
  call CLARGC (9, hcopy, irc)
  if ( irc /= 0 ) hcopy='no'
  call CLARGC (10, hdev, irc)
  if ( hdev(1:4) == 'lips' ) then
     hdev='/lips2'
  else 
     hdev='/PS'
  end if
  call CLARGC (11, hfile, irc)
  if ( irc /= 0 ) then
     if ( hdev == '/lips2' ) then
        hfile='unitmag.lps'
     else
        hfile='unitmag.ps'
     end if
  end if
  call CLARGC (12, pid, irc)
  if ( irc /= 0 ) pid='yes'
  
  allocate( G(iext, jext) )   ! ptr1=malloc(iext*jext*4)
  call MATRD(im,G,iext,jext,irc)
  
  call FTGIOU(nim,irc)
  iblksize=1
  inquire (file='umag_tmp', exist=there)
  if ( there ) call system('/bin/rm umag_tmp ')
  call FTINIT(nim,'umag_tmp',iblksize,irc)
  morekeys=0
  call FTCOPY(im,nim,morekeys,irc)
  bitpix=-32
  call FTUKYJ(nim,'BITPIX',bitpix,' ',irc)
  call FTCPDT(im,nim,irc)
  if ( irc /= 0 ) goto 999
  !       *** SET APERTURE RANGE ***
  AMAX=(MIN(CENTI-1.0,CENTJ-1.0, REAL(iext)-CENTI,REAL(jext)-CENTJ)-5.0) &
       & *PSIZE*2.0
  AMIN=5.0*PSIZE
  
  !     *** INPUT AVAILABLE PHOTOELECTRIC PHOTOMETRY DATA ***
  call CALDAT(AP,AMAG,SOURCE,NAP,lin,ctlg,band)
  
  call adhstry(nim,'UNITMAG',7,'ABSOLUTE CALIBRATION WAS DONE',irc)
  write(comments,'(''Center for UNITMAG = '',f8.2,f8.2)') CENTI, CENTJ
  call adhstry(nim,'UNITMAG',0,comments,irc)
  do j=1, nap
     write(comments,'(''No.'',i3,2f10.3,2x,a10)') j,ap(j),amag(j),source(j)
     call adhstry(nim,'UNITMAG',0,comments,irc)
  end do
  
  !       *** CREATE GROWTH CURVE ***
  RMAX=MIN(CENTI-1.0,CENTJ-1.0,REAL(iext)-CENTI,REAL(jext)-CENTJ)
  KSTEP=500
  call CALGRC(G,iext,jext,CENTI,CENTJ,PSIZE,RMAX,GROW,KSTEP)
  
  !     *** PERFORM LEAST SQUARES FITTING ***
  call CALFIT(TITLE,AP,AMAG,DMAG,REJECT,NAP,GROW,KSTEP, &
       &     rng,RMAX,PSIZE,AMAX,AMIN,ACMAX,CFACT, &
       &     LXMIN,LXMAX,LYMIN,LYMAX,LXINT,LYINT,SKY,SIGMA, &
       &     scale,device,hcopy,hdev,hfile,pid)
  
  call CALPRN(AP,AMAG,DMAG,REJECT,SOURCE,NAP,SKY,SIGMA,TITLE,ACMAX,CFACT,nlp)
  
  call FTPKYF(nim,'UNIT-MAG',sky,4,'UNIT OF BRIGHTNESS (mag/arcsec**2)',irc)
  call FTPKYF(nim,'UMAG-ERR',sigma,4,'ERROR IN UNIT OF BRIGHTNESS',irc)
  
  call MATWT (nim,G,iext,jext,irc)
  
  call FTCLOS(nim,irc)
  if (irc /= 0) go to 999
  call system('/bin/mv umag_tmp '//image)
  
9000 deallocate(G) 
  !     call PGEND
  STOP
  
999 close(lin)
  call PUTEMSG(irc)
  deallocate(G) 
  stop
  
end program unitmag

!-----------------------------------------------------------------------------
SUBROUTINE CALDAT(AP,AMAG,SOURCE,NAP,lin,ctlg,band)

  !     *** INPUT PHOTOELECTRIC PHOTOMETRY DATA ***

  character SOURCE(300)*10,YN1,YN2,FNAME*30,DUMMY*30
  character ctlg*3, band
  dimension AP(300),AMAG(300)

  NAP=0
  yn1='y'
  if ( ctlg == 'no' ) yn1='n'
  if ( yn1 /= 'n' ) then
     call LONGO (AP,AMAG,SOURCE,NAP,lin,irc,band)
     if ( irc == 1 ) go to 1000
     return
  end if
  
1000 continue
  print *
  print '(''Use data-file of p.e. magnitude? ( <y>/n ) ===> '', $)'
  read '(A1)',  YN1
  if(yn1 == 'n') then

     !       *** DATA INPUT FROM TERMINAL ***
2000 continue
     print *
     print *, 'Enter photoelectric photometry data        '
     do I=NAP+1,300
        print *
        print *, '     aperture |magnitude|  source  '
        print *, '     (arcsec) |  (mag)  | (all blank for end)'
        print *, '     ----+----|----+----|----+----|'
        print '(''===> '',$)'
        read '(A30)', DUMMY
        if (DUMMY == '                              ') go to 200
        read (DUMMY( 1:10),'(BN,F10.0)') AP(I)
        if (AP(I) <= 0.0) go to 200
        read (DUMMY(11:20),'(BN,F10.0)') AMAG(I)
        SOURCE(I)=DUMMY(21:30)
     end do
     I=301
200  continue
     NAP=I-1

     !       *** DISPLAY DATA ***
300  continue
     do I=1,NAP
        print '(I3,2X,F7.2,5X,F7.2,5X,A10)', I,AP(I),AMAG(I),SOURCE(I)
     end do
     print *
     print '(''Display again?  ( y/<n> ) ===> '',$)'
     read '(A1)', YN2
     if(yn2 == 'y') go to 300

     !       *** CHANGE DATA ***
     print *
     print *, 'Enter data seq.number to be changed       '
     print *, '    ( 0; no change / -1; more data input )'
     print '(''Enter number ===> '',$)'
     read *, ICH
     if (ICH < 0) then
        go to 2000
     else if (ICH > 0.and.ICH <= NAP) then
        print *, '--- THE DATA TO BE CHANGED ---'
        print '(I3,2X,F7.2,5X,F7.2,5X,A10)', &
             & ICH,AP(ICH),AMAG(ICH),SOURCE(ICH)
        print *, '     ----+----|----+----|----+----|'
        print '(''===> '',$)'
        read '(A30)', DUMMY
        read (DUMMY( 1:10),'(BN,F10.0)') AP(ICH)
        read (DUMMY(11:20),'(BN,F10.0)') AMAG(ICH)
        SOURCE(ICH)=DUMMY(21:30)
        if (DUMMY == '                              '.OR. &
             &  AP(ICH) <= 0.0)       then
           do I=ICH+1,NAP
              AP(I-1)=AP(I)
              AMAG(I-1)=AMAG(I)
              SOURCE(I-1)=SOURCE(I)
           end do
           NAP=NAP-1
        end if
        go to 300
     end if
     !
  else
     !
     !       *** DATA INPUT FROM FILE ***
     print '(''  Enter file name ( ex.  cal.b  ) ===> '',$)'
     read (*,'(A30)') FNAME
     
     OPEN (12,FILE=FNAME,STATUS='OLD',ACCESS='SEQUENTIAL',FORM='formatTED')
     do I=1,300
        read (12,'(A30)',END=100) DUMMY
        read (DUMMY( 1:10),'(BN,F10.0)') AP(I)
        read (DUMMY(11:20),'(BN,F10.0)') AMAG(I)
        SOURCE(I)=DUMMY(21:30)
     end do
     I=301
100  continue
     NAP=I-1
     CLOSE(12)
  end if

  return
end SUBROUTINE CALDAT

!---------------------------------------------------------------------
SUBROUTINE CALGRC(G,iext,jext,CENTI,CENTJ,PSIZE,RMAX,GROW,KSTEP)

  !     *** COMPUTE GROWTH CURVE ***

  dimension G(iext,jext),GROW(0:KSTEP)

  !     *** DEFINE INNER REGION ***
  IS=MAX(   1,INT(CENTI-20.0))
  IE=MIN(iext,INT(CENTI+20.0))
  JS=MAX(   1,INT(CENTJ-20.0))
  JE=MIN(jext,INT(CENTJ+20.0))

  do N=0,KSTEP
     GROW(N)=0.0
  end do

  !     *** Sort pixel in outer region ***
  call CALGR1(G,iext,jext,  1,   1,iext,JS-1, CENTI,CENTJ,RMAX,GROW,KSTEP)
  call CALGR1(G,iext,jext,  1,JS  ,IS-1,JE  , CENTI,CENTJ,RMAX,GROW,KSTEP)
  call CALGR1(G,iext,jext,IE+1,JS ,iext,JE  , CENTI,CENTJ,RMAX,GROW,KSTEP)
  call CALGR1(G,iext,jext,  1,JE+1,iext,jext, CENTI,CENTJ,RMAX,GROW,KSTEP)

  !     *** Use sub-pixel in inner region ***
  RMAX2=RMAX**2
  do J=JS,JE
     do I=IS,IE
        do JJ=-2,2
           RJ=(J+JJ*0.2-CENTJ)**2
           do II=-2,2
              RI=(I+II*0.2-CENTI)**2
              if (RI+RJ <= RMAX2) then
                 NR=SQRT(RI+RJ)/RMAX*KSTEP+1.0
                 if (NR <= KSTEP) then
                    GROW(NR)=GROW(NR)+G(I,J)*0.04
                 end if
              end if
           end do
        end do
     end do
  end do

  !     *** CREATE GROWTH CURVE ***
  do N=2,KSTEP
     GROW(N)=GROW(N)+GROW(N-1)
  end do

  !     *** NORMALIZATION ***
  do N=1,KSTEP
     GROW(N)=GROW(N)*PSIZE*PSIZE
  end do
  !
  return
end SUBROUTINE CALGRC
!---------------------------------------------------------------------
SUBROUTINE CALGR1(G,iext,jext,I1,J1,I2,J2,CENTI,CENTJ,RMAX,GROW,KSTEP)

  !     *** Assistant SUBROUTINE of growth (sort pixel) ***

  dimension G(iext,jext), GROW(0:KSTEP)

  RMAX2=RMAX**2
  do J=J1,J2
     RJ=(J-CENTJ)**2
     do I=I1,I2
        RI=(I-CENTI)**2
        if (RI+RJ <= RMAX2) then
           NR=SQRT(RI+RJ)/RMAX*KSTEP+1.0
           if (NR <= KSTEP) then
              GROW(NR)=GROW(NR)+G(I,J)
           end if
        end if
     end do
  end do

  return
end  SUBROUTINE CALGR1
!----------------------------------------------------------------------
SUBROUTINE CALFIT(TITLE,AP,AMAG,DMAG,REJECT,NAP,GROW,KSTEP, &
     &      rng,RMAX,PSIZE,AMAX,AMIN,ACMAX,CFACT, &
     &      LXMIN,LXMAX,LYMIN,LYMAX,LXINT,LYINT,SKY,SIGMA, &
     &      scale,device,hcopy,hdev,hfile,pid)
  
  !     *** INTERACTIVE FITTING ***
  !              REJECTION SYMBOL   S : TOO SMALL APERTURE
  !                                 L : TOO LARGE APERTURE
  !                                 E : ERROR
  !                                 * : REJECT WITH CURSOR
  !                                 # : REJECT FROM TERMINAL
  !
  character :: REJECT(NAP),YN,mode,cstr*47,TITLE*40,hfile*40,rng*5
  character :: device*10, hcopy*5, scale*5, hdev*6, pid*5
  integer   :: hard
  real(4)   :: AP(NAP),AMAG(NAP),DMAG(NAP),GROW(0:KSTEP),WG(0:500)
  logical   :: there
  
  !     *** INITIALIZE GRAPHIC DISPLAY ***
  print *
  hard=0
  if ( device == 'inq' ) then
     call PGBEG(0,'?/INQ',1,1)
  else
     call PGBEG(0,device,1,1)
  end if
  
  !     *** INITIALIZATION ***
2000 continue
  do N=0,KSTEP
     WG(N)=GROW(N)
  end do
  do N=1,NAP
     REJECT(N)=' '
  end do
  
  !     *** SET MANUAL REJECTION ***
1000 continue
  ACMAX=0.0
  CFACT=1.0
  yn='n'
  if ( rng == 'yes' ) yn='y'
  if (yn == 'y') then
     print '(''Enter rejection aperture  min,max (arcsec) ===> '', $)'
     read *, RAMIN,RAMAX
     do N=1,NAP
        if (AP(N) >= RAMIN.and.AP(N) <= RAMAX.and.REJECT(N) == ' ') &
             & REJECT(N)='#'
     end do
  end if
  
  !     *** COMPUTE INTEGRATION MAGNITUDE ***
  call INTEG1(AP,REJECT,DMAG,NAP,WG,KSTEP,RMAX,PSIZE,AMAX,AMIN)
  
  !     *** LEAST-SQUARES FIT ***
  call SQFIT(AMAG,DMAG,NAP,REJECT,SKY,SIGMA)
  print *
  print '('' Sky = '',f6.3,'' +- '',f6.3)', sky,sigma
  write(cstr,'(''sky = '',f6.3,'' +- '',f6.3, &
       &       '' (mag/arcsec\\\\u2\\\\d)'')') sky,sigma
  
  !     *** DISPLAY-DATA SCALING ***
  XMIN= 0.0
  XMAX= 0.0
  YMAX= 0.0
  YMIN=50.0
  do N=1,NAP
     if (REJECT(N) == ' ') then
        if (AP(N)       > XMAX) XMAX=AP(N)
        if (AMAG(N)     > YMAX) YMAX=AMAG(N)
        if (AMAG(N)     < YMIN) YMIN=AMAG(N)
        if (DMAG(N)+SKY > YMAX) YMAX=DMAG(N)+SKY
        if (DMAG(N)+SKY < YMIN) YMIN=DMAG(N)+SKY
     end if
  end do
  LXMIN=0
  LXMAX=XMAX+(XMAX-XMIN)*0.2
  LYMIN=YMIN-0.5
  LYMAX=YMAX+1.5
  mode='a'
  if ( scale == 'no' ) mode='m'
  if( mode == 'm' ) then
     print '(''Enter max. aperture ===> '',$)'
     read *, DUM1
     KXMAX=DUM1+0.5
     print '(''Enter magnitudes (bright, faint) ===> '',$)'
     read *, DUM1,DUM2
     KYMIN=DUM1+0.5
     KYMAX=DUM2+0.5
     if (REAL(KXMAX) >= XMAX) LXMAX=KXMAX
     if (REAL(KYMIN) <= YMIN) LYMIN=KYMIN
     if (REAL(KYMAX) >= YMAX) LYMAX=KYMAX
  end if
  
  WW=10.0**INT(LOG10(REAL(LXMAX-LXMIN))-1.0)
  KW=NINT(REAL(LXMAX-LXMIN)/WW)
  if (KW >= 50) then
     LXINT=NINT(WW*10.0)
  else if (KW >= 15) then
     LXINT=NINT(WW* 5.0)
  else
     LXINT=NINT(WW* 1.0)
  end if
  LYINT=1
  
  !     *** DISPLAY FITTING RESULTS ***
  print *
  xxmin=LXMIN
  xxmax=LXMAX
  yymin=LYMIN
  yymax=LYMAX
  XSCALE=(xxmax-xxmin)/70.0
  YSCALE=(yymax-yymin)/50.0
  !3000 call PGEND
3000 continue
  call PGENV(xxmin,xxmax,yymax,yymin,-2,-2)
  if ( hard == 0 ) then
     call PGVSTAND
  else
     call PGVPORT(0.15,0.85,0.15,0.85)
  end if
  call PGBOX('BCNST',0.0,-1,'BCNSTV',0.0,-1)
  
  call PGLABEL('Aperture (arcsec)','Integrated magnitude (mag)', TITLE)
  
  if ( hard == 1 .and. pid /= 'no' ) call PGIDEN
  call PGSCH(1.0)
  xdum=xxmin+xscale*30.0
  ydum=yymin+yscale*35.0
  call PGTEXT(xdum,ydum,cstr)
  xdum=xxmin+xscale*40.0
  ydum=ydum+yscale*4.0
  if ( hard == 1 ) call PGSCH(1.4)
  call PGPT(1,xdum,ydum-yscale*0.3,17)
  if ( hard == 1 ) call PGSCH(1.0)
  call PGTEXT(xdum+xscale,ydum,'adopted')
  ydum=ydum+yscale*2.5
  !     if ( hard == 1 ) call PGSCH(0.7)
  call PGPT(1,xdum,ydum-yscale*0.3,5)
  !     if ( hard == 1 ) call PGSCH(1.0)
  call PGTEXT(xdum+xscale,ydum,'rejected (limit)')
  ydum=ydum+yscale*2.5
  !     if ( hard == 1 ) call PGSCH(0.7)
  call PGPT(1,xdum,ydum-yscale*0.3,4)
  !     if ( hard == 1 ) call PGSCH(1.0)
  call PGTEXT(xdum+xscale,ydum,'rejected (cursor)')
  ydum=ydum+yscale*2.5
  !     if ( hard == 1 ) call PGSCH(0.7)
  call PGPT(1,xdum,ydum-yscale*0.3,7)
  !     if ( hard == 1 ) call PGSCH(1.0)
  call PGTEXT(xdum+xscale,ydum,'rejected (term)')
  
  call GPLOT(WG,KSTEP,RMAX,PSIZE,SKY,LXMIN,LXMAX,LYMIN,LYMAX,&
       &     AP,AMAG,NAP,REJECT,hard)
  
  !     *** REJECTION OR OTHER PROCESS ***
  if( hard == 0 ) then
     call MOUSEC(AP,AMAG,NAP,REJECT,XSCALE,YSCALE,irc)
     if ( irc==100 ) go to 100
     if ( irc==200 ) go to 200
     if ( irc==300 ) go to 100
     if ( irc==400 ) go to 400
  end if
  !
  !     *** FITTING end ***
  if ( hard == 0 ) then
     !	 call tidle
     !	 call mgoclose
     call PGEND
     !	 call tsetup
     if ( hcopy == 'yes' ) hard=1
     if ( hard == 1 ) then
        if ( hdev == '/lips2' ) then
           inquire (file=hfile, exist=there)
           if ( there ) call system('/bin/rm '//hfile//' >/dev/null')
        end if
        call PGBEG(0,hfile//hdev,1,1)
        call PGSCF(2)
        go to 3000
     end if
  end if
  if ( hard == 1 ) then
     !----    Finish the plot
     call PGEND
     if ( hdev == '/lips2' ) then
        print *, '### Output(LIPS2) was written in '//hfile
     else
        print *, '### Output(PostScript) was written in '//hfile
     end if
  end if
  
  do N=0,KSTEP
     GROW(N)=WG(N)
  end do
  return
  
  !     *** RETRY ***
100 continue
  go to 1000
  
  !     *** INITIALIZE ***
200 continue
  go to 2000
  
  !     *** CORRECTION OF GROWTH CURVE ***
400 continue
  print *
  print *, '*** Correction of growth curve ***        '
  print *, ' in( aperture < acmax )---> growth*cfact  '
  print '(''Enter acmax  ===> '',$)'
  read *, ACMAX
410 continue
  print '(''Enter cfact  ===> '',$)'
  read *, CFACT
  if (CFACT < 0.0.OR.CFACT > 100.0) go to 410
  call CALCOR(WG,KSTEP,RMAX,PSIZE,ACMAX,CFACT)
  go to 1000
  !
end SUBROUTINE CALFIT

!----------------------------------------------------------------------
SUBROUTINE INTEG1(AP,REJECT,DMAG,NAP,GROW,KSTEP,RMAX,PSIZE,AMAX,AMIN)

  !     *** COMPUTE INTEGRATION MAGNITUDE BY INTERPOLATION ***
  
  dimension AP(NAP),DMAG(NAP),GROW(0:KSTEP)
  character REJECT(NAP)

  ASTEP=RMAX/KSTEP*PSIZE

  do N=1,NAP
     if (AP(N) > AMAX) then
        REJECT(N)='L'
        DMAG(N)=111.111
     else if (AP(N) < AMIN) then
        REJECT(N)='S'
        DMAG(N)=111.111
     else
        NSTEP=AP(N)/ASTEP*0.5
        if (NSTEP >= KSTEP) then
           REJECT(N)='L'
           DMAG(N)=111.111
        else
           R1=AP(N)*0.5-NSTEP*ASTEP
           R2=ASTEP
           R3=GROW(NSTEP+1)-GROW(NSTEP)
           R4=GROW(NSTEP)
           D=R1/R2*R3+R4
           if (D > 0.0) then
              DMAG(N)=-2.5*LOG10(D)
           else
              DMAG(N)=111.11
              REJECT(N)='E'
           end if
        end if
     end if
  end do
  
  return
end SUBROUTINE INTEG1

!----------------------------------------------------------------------
SUBROUTINE SQFIT(AMAG,DMAG,NAP,REJECT,SKY,SIGMA)
  
  !     *** LEAST-SQUARES FIT ***

  dimension AMAG(NAP),DMAG(NAP)
  character REJECT(NAP)
  REAL*8    S,SS
  
  N=0
  S =0.0D0
  SS=0.0D0
  do I=1,NAP
     if (REJECT(I) == ' ') then
        N=N+1
        S =S +(AMAG(I)-DMAG(I))
        SS=SS+(AMAG(I)-DMAG(I))**2
     end if
  end do
  SKY=S/REAL(N)
  if (N >= 2) then
     SIGMA=SQRT((SS-S*S/DBLE(N))/DBLE(N-1))
  else
     SIGMA=-1.0
  end if
  !
  return
end SUBROUTINE SQFIT

!----------------------------------------------------------------------
SUBROUTINE GPLOT(WG,KSTEP,RMAX,PSIZE,SKY,LXMIN, & 
     &     LXMAX,LYMIN,LYMAX,AP,AMAG,NAP,REJECT,hard)
  
  !     *** PLOT GROWTH CURVE ***

  character REJECT(NAP)
  integer	hard
  dimension WG(0:KSTEP)
  dimension XP(500),YP(500)
  dimension AP(NAP),AMAG(NAP)
  
  !     *** PLOT DATA POINT ***
  STEP=RMAX/KSTEP*PSIZE*2.0
  NEFF=0
  do N=1,KSTEP
     A=REAL(N)*STEP
     B=-2.5*LOG10(WG(N))+SKY
     if (A >= REAL(LXMIN).and.A <= REAL(LXMAX).and. &
          &     B >= REAL(LYMIN).and.B <= REAL(LYMAX))  then
        NEFF=NEFF+1
        XP(NEFF)=a
        YP(NEFF)=b
     end if
  end do
  call PGLINE(neff,xp,yp)
  
  !     *** PLOT PHOTOELECTRIC PHOTOMETRY DATA***
  NEFF=0
  do N=1,NAP
     if (REJECT(N) == ' ') then
        NEFF=NEFF+1
        XP(NEFF)= AP(N)
        YP(NEFF)= AMAG(N)
     end if
  end do
  if ( hard == 1 ) then
     call PGSCH(1.4)
     call PGPT(neff,xp,yp,17)
     call PGSCH(1.0)
  else
     call PGSCH(1.4)
     call PGPT(neff,xp,yp,17)
     call PGSCH(1.0)
  end if
  NEFF=0
  do N=1,NAP
     if (REJECT(N) == '*') then
        NEFF=NEFF+1
        XP(NEFF)= AP(N)
        YP(NEFF)= AMAG(N)
     end if
  end do
  if ( hard == 1 ) then
     !        call PGSCH(0.7)
     call PGPT (neff,xp,yp,4)
     !        call PGSCH(1.0)
  else
     call PGPT (neff,xp,yp,4)
  end if
  NEFF=0
  do N=1,NAP
     if (REJECT(N) == 'S' .or. REJECT(N) == 'L' ) then
        NEFF=NEFF+1
        XP(NEFF)= AP(N)
        YP(NEFF)= AMAG(N)
     end if
  end do
  if ( hard == 1 ) then
     !        call PGSCH(0.7)
     call PGPT(neff,xp,yp,5)
     !        call PGSCH(1.0)
  else
     call PGPT(neff,xp,yp,5)
  end if
  NEFF=0
  do N=1,NAP
     if (REJECT(N) == '#' ) then
        NEFF=NEFF+1
        XP(NEFF)= AP(N)
        YP(NEFF)= AMAG(N)
     end if
  end do
  if ( hard == 1 ) then
     !        call PGSCH(0.7)
     call PGPT(neff,xp,yp,7)
     !        call PGSCH(1.0)
  else
     call PGPT(neff,xp,yp,7)
  end if
  !     call tidle
  
  return
end SUBROUTINE GPLOT

!----------------------------------------------------------------------
SUBROUTINE MOUSEC(AP, AMAG, NAP, REJECT, XSCALE, YSCALE, irc)

  !     *** REJECTION OR OTHER PROCESS ***
  
  character REJECT(NAP),ANS
  dimension AP(NAP),AMAG(NAP)
  dimension X(300),Y(300)

  irc=0
  !     *** DISPLAY MENU ***
  print '('' '')'
  print '(''<1>: OK        '')'
  print '('' 2 : Replot    '')'
  print '('' 3 : Initialize'')'
  print '('' 4 : Rejection '')'
  print '('' 5 : Correction'')'
  !     *** SELECT MENU ***
  print '(''             ===> '',$)'
  read '(a)', ans

  !------------------------------------
  select case (ans)
  case('2') 
     irc=2
     return
  case('3')
  case default
  endselect
  
!-----------------------------------
  if ( ans == '' ) then
     idata=1
  else
     read(ans,*) idata
  end if
  !     read (*,*) idata

  if (IDATA == 2) then
     NEXT=2
  else if (IDATA == 3) then
     NEXT=3
  else if (IDATA == 4) then
     NEXT=4
  else if (IDATA == 5) then
     NEXT=5
  else
     NEXT=1
  end if
  
  !     *** DETERMINE NEXT PROCESS ***
  go to (100,200,300,400,500) NEXT

  !     *** FITTING end ***
100 continue
  return

  !     *** RETRY ***
200 continue
  irc=100
  return 

  !     *** INITIALIZE ***
300 continue
  irc=200
  return 

  !     *** REJECTION ***
400 continue
  print *
  print *, '*** REJECTION ***'
  XRANGE=XSCALE/2.0
  YRANGE=YSCALE/2.0
  !     call cursin(X,Y,NAP,NREJ)
  call curs_in(X,Y,NAP,NREJ)
  do I=1,NREJ
     XP= X(I)
     YP= Y(I)
     do N=1,NAP
        if (REJECT(N) == ' ') then
           RX=ABS(XP-AP(N))
           RY=ABS(YP-AMAG(N))
           if (RX <= XRANGE.and.RY <= YRANGE) then
              REJECT(N)='*'
              print *, 'REJECT  NO.', N
              go to 10
           end if
        end if
     end do
10 continue
  end do
  irc=300
  return 

  !     *** CORRECTION ***
500 continue
  irc=400
  return 

end SUBROUTINE MOUSEC

!---------------------------------------------------------------------
SUBROUTINE CALCOR(WG,KSTEP,RMAX,PSIZE,ACMAX,CFACT)
  
  !     *** CORRECTION OF GROWTH CURVE ***
  
  dimension WG(0:KSTEP)
  
  STEP=RMAX/KSTEP*PSIZE*2.0
  NMAX=ACMAX/STEP
  ACINT=WG(NMAX)*(CFACT-1.0)
  
  do N=1,NMAX
     WG(N)=WG(N)*CFACT
  end do
  
  do N=NMAX+1,KSTEP
     WG(N)=WG(N)+ACINT
  end do
  
  return

end SUBROUTINE CALCOR

!----------------------------------------------------------------------
SUBROUTINE CALPRN(AP,AMAG,DMAG,REJECT,SOURCE, &
     &     NAP,SKY,SIGMA,TITLE,ACMAX,CFACT,nlp)

  !       *** PRINT RESULTS ON nlp ***

  dimension AP(NAP),AMAG(NAP),DMAG(NAP)
  character SOURCE(NAP)*10,REJECT(NAP),TITLE*40

  write(nlp,'(//1H ,10X,"ABSOLUTE CALIBLATION"/ 1H ,15X,"Title : ",A40)') &
       &     TITLE
  write(nlp,'(1H ,30X,"Reject  L : Aperture is too large"/ &
         &       1H ,30X,"        S : Aperture is too small"/ &
         &       1H ,30X,"        # : Manual rejection     "/ &
         &       1H ,30X,"        * : Cursor rejection     "/)')
  if (ACMAX > 1.0E-5) &
     write(nlp,'(1H ,20X,"Correction of growth curve"/ &
          &       1H ,20X,"     Aperture < ",F7.2," arcsec"/ &
          &       1H ,20X,"     Factor   = ",F7.2)') ACMAX,CFACT
  write(nlp,'(1H ,7X,"Aperture",4X,"Ph.el.mag ",2X,"Integ.mag ", &
       &      4X,"Reject",2X,"Source")')
  do I=1,NAP
     write(nlp,'(1H ,2X,I3,2X,3(F7.3,5X),4X,A1,5X,A10)') &
          &   I,AP(I),AMAG(I),DMAG(I),REJECT(I),SOURCE(I)
  end do
  if (SIGMA >= 0.0) then
     write(nlp,'(/1H ,20X,"Sky = ",F7.3," +- ",F7.3)')  SKY,SIGMA
  else
     write(nlp,'(/1H ,20X,"SKY = ",F7.3)') sky
  end if

  return

end SUBROUTINE CALPRN

!---------------------------------------------------------------------------
SUBROUTINE LONGO(AP,AMAG,SOURCE,NAP,lin,irc,band)
  
  character SOURCE(300)*10,AUTHOR*10,BAND
  character OBJECT*12, GALAXY*12, SRC*8, NOTE*5, toupper*12
  character CBV*5, CUB*5, FV, FBV, FUB
  dimension AP(300), AMAG(300)
  
  irc=0
  
  print *
  print *, 'Enter galaxy name  '
  print *, '  e.g.,  N 205, N4321, I1613, A1234-56, etc. )'
  print '(''    ===> '',$)'
  read '(A12)', OBJECT
  object=toupper(object)
  do J=1,100000
     read (lin,'(A12)',END=999) GALAXY
     if ( GALAXY == OBJECT ) go to 1000
  end do
  go to 999
1000 backspace lin
  NAP=0
  print *
  print '(''Color band: '',a1)', band
  if ( band == 'u' ) then
     do N=1,1000
        read (lin,'(A12,14X,F5.0,1X,A8,1X,F5.0,A1,1X,A5,A1,1X,A5,A1,A5)') &
             &      GALAXY,A,SRC,V,FV,CBV,FBV,CUB,FUB,NOTE
        if ( GALAXY /= OBJECT ) go to 2000
        !       if ( cub == '     ' .or. fv == 'b' ) then      go to 10
        if ( cub /= '     ' .and. fv /= 'b') then
           !       else
           read (CBV,'(F5.0)') BV
           read (CUB,'(F5.0)') UB
           U=UB+BV+V
        end if
        A=10.0**A*6.0
        if ( FV == '*' .OR. FBV == '*' .OR. FUB == '*' ) then 
           AUTHOR=SRC//' *'
        else
           AUTHOR=SRC//'  '
        end if
        print '('' NO.'',I2,5X,F9.3,2X,F7.3,2X,A10,A5)', N, A, U, AUTHOR,NOTE
        NAP=NAP+1
        AP(NAP)=A
        AMAG(NAP)=U
        SOURCE(NAP)=AUTHOR
     end do
  else if( band == 'b' ) then
     print *
     do N=1,1000
        read (lin,'(A12,14X,F5.0,1X,A8,1X,F5.0,A1,1X,A5,A1,1X,A5,A1,A5)') &
             &                  GALAXY,A,SRC,V,FV,CBV,FBV,CUB,FUB,NOTE
        if ( GALAXY /= OBJECT ) go to 2000
        !       if ( V == 0.0 ) go to 20
        if ( V /= 0.0 ) then
           if ( FV == 'B' ) then
              B=V
              !          else if ( CBV == '     ' ) then
           else if ( CBV /= '     ' ) then
              !             go to 20
              !          else
              read (CBV,'(F5.0)') BV
              B=V+BV
           end if
           A=10.0**A*6.0
           if ( FV == '*' .OR. FBV == '*' ) then
              AUTHOR=SRC//' *'
           else
              AUTHOR=SRC//'  '
           end if
           print '('' NO.'',I2,5X,F9.3,2X,F7.3,2X,A10,A5)', N, A, B, &
                &     AUTHOR, NOTE
           NAP=NAP+1
           AP(NAP)=A
           AMAG(NAP)=B
           SOURCE(NAP)=AUTHOR
        end if
     end do
  else
     print *
     do N=1,1000
        read (lin,'(A12,14X,F5.0,1X,A8,1X,F5.0,A1,1X,A5,A1,1X,A5,A1,A5)') &
             &                  GALAXY,A,SRC,V,FV,CBV,FBV,CUB,FUB,NOTE
        if ( GALAXY /= OBJECT ) go to 2000
        AUTHOR=SRC//'  '
        if ( FV == '*' ) AUTHOR=SRC//' *'
        !        if ( V == 0.0 .OR. FV == 'B' ) go to 30
        if ( V /= 0.0 .and. FV /= 'B' ) then
           A=10.0**A*6.0
           print '('' NO.'',I2,5X,F9.3,2X,F7.3,2X,A10,A5)', &
                & N, A, V, AUTHOR, NOTE
           NAP=NAP+1
           AP(NAP)=A
           AMAG(NAP)=V
           SOURCE(NAP)=AUTHOR
        end if
     end do
  end if
  !
2000 continue
  return
  
999 print *
  print *, ' Photometric data do not exist!'
  irc=1
  return

end SUBROUTINE LONGO

