!**********************************************************************
!	subroutine les1   (x,y,a,b,n,err,EA,EB)
!	subroutine curfit (x,y,sgy,npts,nterms,mode,a,da,
!    +			   sga,flamda,yfit,chisqr)
!	subroutine matinv (a,n,det)
!	subroutine wgtcal (weight,y,sgy,npts,mode)
!	SUBROUTINE FITELL (X,Y,XC,YC,N,IC,RMAJ,RMIN,ANG,SIGM,IERR)
!	SUBROUTINE FITE   (X,Y,      ND,  RMAJ,RMIN,ANG,SIGM,IERR)
!       SUBROUTINE LEAST  (TABOBS,NDIM,NARAY,NTOTAL,FACTOR,SIGMA,ier)
!       SUBROUTINE LNSYS  (ARRAY,X,NORDER,ier)
!       Modified by M. Hamabe 2007-10-16 (for Fortran90)
!**********************************************************************
subroutine les1(x,y,a,b,n,err,EA,EB)
!	This is a very simple linear least-squares fitting routine.
!	Errors of the obtained coefficients are also calculated.

  parameter (MAXPNT=500)
  implicit real*8 (A-H,O-Z)
  real*4     X(MAXPNT),Y(MAXPNT),A,B,ERR,EA,EB
  dimension xx(MAXPNT),yy(MAXPNT)

  tx =0.0D0
  ty =0.0D0
  txy=0.0D0
  txx=0.0D0
  do  i=1,n
     xx(i)=x(i)
     yy(i)=y(i)
     tx  = tx + xx(i)
     ty  = ty  + yy(i)
     txx = txx + xx(i)*xx(i)
     txy = txy + xx(i)*yy(i)
  end do
  AX=TX/real(N)
  AY=TY/real(N)
  SXY = txy - N*AX*AY
  SXX = txx - N*AX*AX
  if ( SXX==0.0 ) print *, ' N=',N
  aa = SXY / SXX
  bb = AY - aa*AX
  
  sum=0.0D0
  do  i=1,n
     dum = yy(i)-( aa*xx(i)+bb )
     sum = sum + dum*dum
  end do
  a=aa
  b=bb
  wt=n-2
  if ( wt<1.0 ) wt=1.0D0
  VAR = sum/wt
  err = sqrt(VAR)
  EA = sqrt( VAR/SXX )
  EB = sqrt( VAR*(1.0/N+AX*AX/SXX) )
  
  return

end subroutine les1

!***********************************************************
subroutine curfit (x,y,sgy,npts,nterms,mode,a,da,sga,flamda,yfit,chisqr)
  
  parameter (MAXPNT=500)
  real*8	  array(10,10),weight(MAXPNT),ALPHA(10,10)
  dimension x(1),y(1),sgy(1),a(4),da(1),sga(1),yfit(1)
  dimension beta(10),deriv(10),b(10)
  
  nfree = npts - nterms
  if ( nfree<=0 ) then
     chisqr = 0.0
     return
  end if
!
!	evaluate weight
!
  call wgtcal (weight,y,sgy,npts,mode)
!
!		evaluate alpha and beta matrix
!
  do  j=1,nterms
     beta(j) = 0.0
     do  k=1, j
        alpha(j,k) = 0.0
     end do
  end do
  do  i=1, npts
     call fderiv(x,i,a,da,nterms,deriv)
     do  j=1,nterms
        beta(j) = beta(j) +weight(i)*(y(i)-galaxy(x,i,a))*deriv(j)
        do  k=1, j
           alpha(j,k) = alpha(j,k) + weight(i)*deriv(j)*deriv(k)
           if ( WEIGHT(I)==0.0 ) print *, '0-WEIGHT'
        end do
     end do
  end do
  do  j=1,nterms
     do  k=1, j
        alpha(k,j) = alpha(j,k)
     end do
  end do
!
!		evaluate chi square at starting point
!
  do  i=1, npts
     yfit(i) = galaxy(x,i,a)
  end do
  chisq1 = fchisq (y,sgy,npts,nfree,mode,yfit)
!
!		invert modified curvature matrix to find new parameters
!
1000 do  j=1,nterms
     do  k=1,nterms
        array(j,k) = alpha(j,k) / sqrt(alpha(j,j)*alpha(k,k))
     end do
     array(j,j) = 1.0+flamda
  end do
  call matinv (array, nterms, det)
  do  j=1, nterms
     b(j) = a(j)
     do  k=1, nterms
        b(j) = b(j) +beta(k)*array(j,k)/sqrt(alpha(j,j)*alpha(k,k))
     end do
  end do
  
!		if chi square increased, increase flamda and try again
  
  do i=1,npts
     yfit(i) = galaxy (x,i,b)
  end do
  chisqr = fchisq (y,sgy,npts,nfree,mode,yfit)
  
  if ( chisqr>chisq1 ) then
     flamda = 10.0*flamda
     go to 1000
  end if

!		evaluate parameters and uncertainties
  
  do j=1,nterms
     a(j) = b(j)
     sga(j) = sqrt( array(j,j)/alpha(j,j) )
  end do
  flamda = flamda/10.0
  
  return

end subroutine curfit

!***********************************************************
subroutine matinv (a,n,det)

  real(8) :: a(10,10), amax, save
  integer :: ik(10),jk(10)

  det = 1.0
  
  do  k=1,n

!		find largest a(i,j) in rest of matrix

     amax=0.
1000 do  i=k,n
        do  j=k,n
           if ( abs(amax)<=abs(a(i,j)) ) then
              amax = a(i,j)
              ik(k) = i
              jk(k) = j
           end if
        end do
     end do

!		interchnge rows and columns to put amax in a(k,k)
     
     if ( amax==0 ) then
        det = 0.0
        return
     end if
     i = ik(k)
     if ( i<k ) then
        go to 1000
     else if ( i>k ) then
        do  j=1,n
           save = a(k,j)
           a(k,j) = a(i,j)
           a(i,j) = -save
        end do
     end if
     j = jk(k)
     if ( j<k ) then
        go to 1000
     else if ( j>k ) then
        do  i=1,n
           save = a(i,k)
           a(i,k) = a(i,j)
           a(i,j) = -save
        end do
     end if

!		accumulate elements of inverse matrix
     
     do  i=1, n
        if ( i/=k ) then
           a(i,k) = -a(i,k) /amax
        end if
     end do
     do  i=1, n
        do  j=1, n
           if ( i/=k .and. j/=k ) then
              a(i,j) = a(i,j) + a(i,k)*a(k,j)
           end if
        end do
     end do
     do  j=1, n
        if ( j/=k ) then
           a(k,j) = a(k,j) / amax
        end if
     end do
     a(k,k) = 1.0/amax
     det = det * amax
  end do

!		restore ordering of matrix
  
  do  l=1, n
     k = n - l + 1
     j = ik(k)
     if ( j>k ) then
        do  i=1, n
           save = a(i,k)
           a(i,k) = - a(i,j)
           a(i,j) = save
        end do
     end if
     i = jk(k)
     if ( i>k ) then
        do  j=1, n
           save = a(k,j)
           a(k,j) = -a(i,j)
           a(i,j) = save
        end do
     end if
  end do
  
  return

end subroutine matinv

!************************************************************
subroutine wgtcal (weight,y,sgy,npts,mode)

  real*8 weight(1)
  dimension y(1), sgy(1)
  
  do  i=1, npts
     if( mode<0 ) then
        if ( y(i)>0.0 ) then
           weight(i) = 1.0/y(i)
        else if ( y(i)<0.0 ) then
           weight(i) = 1.0/(-y(i))
        else
           weight(i) = 1.0
        end if
     else if ( mode>0 ) then
        weight(i) = 1.0/sgy(i)**2
     else
        weight(i) = 1.0
     end if
  end do
  
  return

end subroutine wgtcal

!*******************************************************************C
!               SUBROUTINE OF ELLIPSES FITTING                      C
!            IT TAKES THE GENERAL EQUATION THAT AN ELLIPSE          C
!        HAS TO SATISFY AND FIT BY LEAST-SQUARE THE COEFFICIENTS    C
!        IF THE AXIS WERE ALIGNED WITH X AND Y , THE EQUATION       C
!        WOULD BE :   (X-XC)**2/A2 + (Y-YC)**2/B2 = 1               C
!        WITH A CERTAIN ANGLE ALPH THE COORDINATES TRANSFORMS       C
!        INTO :   X' = COS (APLH) * (X-XC) - SIN (ALPH) * (Y-YC)    C
!                 Y' = SIN (ALPH) * (X-XC) + COS (ALPH) * (Y-YC)    C
!        REPLACING IN THE FORMER EQUATION GIVES AN EQUATION WHOSE   C
!        FORM IS :   A (X-XC)**2 + B (Y-YC)**2 + C X*Y = 1          C
!        A VERY SIMPLE CALCULUS GIVES THE ANSWER FOR THE VALUE      C
!        OF THE DIFFERENT PARAMETERS WHEN THE COEFFICIENTS A,B,C    C
!        ARE DETERMINED BY LEAST-SQUARE.                            C
!                                                                   C
!    INPUT :                                                        C
!        X : TABLE OF X COORDINATES                                 C
!        Y : TABLE OF Y COORDINATES                                 C
!        XC,YC : COORDINATES OF CENTER                              C
!        N : NUMBER OF POINTS THAT HAVE TO BE FITTED                C
!        IC : FLAG FOR DETERMINATION OF CENTER (0) OR NO (1)        C
!                                                                   C
!    OUTPUT :                                                       C
!        RMAJ,RMIN,ANG : PARAMETERS OF THE ELLIPSE                  C
!        XC,YC : EVENTUALLY NEW COORDINATES CENTER                  C
!        SIGM,IERR : SIGMA AND ERROR FLAG                           C
!*******************************************************************C
SUBROUTINE FITELL (X,Y,SXC,SYC,N,IC,SRMAJ,SRMIN,SANG,SSIGM,IERR)
  
  parameter (MAXPNT=500)
  implicit real*8  (A-H,O-Z)
  real*4    X(MAXPNT),Y(MAXPNT),SXC,SYC,SRMAJ,SRMIN,SANG,SSIGM
  dimension U(MAXPNT,6),F(5)
  DATA PI/3.141592653589D0/
  
  FACT=180.0D0/PI
  XC=SXC
  YC=SYC
  IERR=0
  if ( IC==0 ) then		
!				!  CENTER HAS TO BE FOUND
     DX=0.0D0
     DY=0.0D0
     do  K=1,N
        U(K,1)=(X(K)-XC)*(X(K)-XC)
        U(K,2)=(Y(K)-YC)*(Y(K)-YC)
        U(K,3)=(X(K)-XC)*(Y(K)-YC)
        U(K,4)=X(K)-XC
        U(K,5)=Y(K)-YC
        U(K,6)=1.0D0
     end do
     call LEAST(U,MAXPNT,6,N,F,SIGM,ier)
     if ( ier==0 ) then
        DUM = 4.0D0*F(1)*F(2) - F(3)*F(3)
        DX  = ( -2.0D0*F(2)*F(4) +       F(3)*F(5) ) / DUM
        DY  = (        F(3)*F(4) - 2.0D0*F(1)*F(5) ) / DUM
     else
        dx  = 0.
        dy  = 0.
        print *, ' Center could not be calculated !'
     end if
     XC  = XC + DX
     YC  = YC + DY
!-------------------     CORRECTS THE FCTRS TO TAKE INTO ACCOUNT THE BAD CEN
     CORREC=F(1)*DX*DX+F(2)*DY*DY
     CORREC=CORREC+F(3)*DX*DY
     COR=1.0D0 - ( CORREC / (1.0D0+CORREC) )
     F(1)=F(1)*COR
     F(2)=F(2)*COR
     F(3)=F(3)*COR
  else 				
!				!  CENTER ALREADY DETERMINED
     do  K=1,N
        U(K,1)=(X(K)-XC)*(X(K)-XC)
        U(K,2)=(Y(K)-YC)*(Y(K)-YC)
        U(K,3)=(X(K)-XC)*(Y(K)-YC)
        U(K,4)=1.
     end do
     call LEAST(U,MAXPNT,4,N,F,SIGM,ier)
  end if
  SXC=XC
  SYC=YC
  SSIGM=SIGM
!--------------------------------C     DETERMINATION OFTHE PARAMETERS     C
  F1=F(1)
  F2=F(2)
  F3=F(3)
!------------------------------------------------   COMPONENT ON x-AXIS
  G = F3*F3 + (F1-F2)*(F1-F2)
  Z = 2.0D0/( F1+F2+sqrt(G) )
  if ( Z<=0.0D0 ) then
     IERR=-1
     return
  end if
  RMAJ=sqrt(Z)
  A=RMAJ
!------------------------------------------------   COMPONENT ON y-AXIS
  Z= F1 + F2 -1.0D0/(RMAJ*RMAJ)
  if ( Z<=0.0D0 ) then
     IERR=-2
     return
  end if
  RMIN=1./sqrt(Z)
  B=RMIN
  IR=0
  if ( RMIN>RMAX ) then
     IR=1
     TT=RMAJ
     RMAJ=RMIN
     RMIN=TT
  end if
!-----------------------------------------    ANGLE
!				  1)    x/HORIZONTAL
!    SIGN- BECAUSE IT IS THE REVERSE THAT WE WANT (FROM NORMAL TO ELLIPSE)
!
  G1= - F3 / ( F1 - F2 + 1.0D0/(A*A) - 1.0D0/(B*B) )
  ARGUM=G1
  ANG=ATAN(ARGUM)*FACT

  if ( IR==1 ) then	
!					! 2)  CHECK IF x IS MAJOR AXIS
     if ( ANG>=0.0D0  ) then
        ANG=ANG-90.0D0
     else
        ANG=ANG+90.0D0
     end if
  end if
  ANG=ANG+90.0D0		
!				! 3)  ANGLE MAJOR AXIS/VERTICAL, UNCLOCKWISE

  SRMIN=RMIN
  SRMAJ=RMAJ
  SANG =ANG
  
  return
END SUBROUTINE FITELL
!*******************************************************************C
SUBROUTINE FITE (X,Y,N,SRMAJ,SRMIN,SANG,SG,IERR)
!
!	This is an ellipse fitting routine provided by Dr.J.Souviron.
!
  parameter (MAXPNT=500)
  implicit real*8 (A-H,O-Z)
  real*4    X(MAXPNT),Y(MAXPNT),SRMAJ,SRMIN,SANG,SG
  dimension U(MAXPNT,6),F(5)
  
  IERR=0
  do  K=1,N
     U(K,1)= X(K)*X(K)
     U(K,2)= Y(K)*Y(K)
     U(K,3)= X(K)*Y(K)
     U(K,4)=1.
  end do
  
  call LEAST(U,MAXPNT,4,N,F,SIGM,ier)
!-----------------------------------------C
!     DETERMINATION OF THE PARAMETERS     C
!-----------------------------------------C
  F1=F(1)
  F2=F(2)
  F3=F(3)
  FACT=180./3.141593
!
!--------------   COMPONENT ON x-AXIS
!
  G1=F3*F3+(F1-F2)*(F1-F2)
  G2=( 2./ ( F1+F2+sqrt(G1) ) )
  Z=G2
  if ( Z<=0. ) then
     IERR=-1
     return
  end if
  RMAJ=sqrt(Z)
  A=RMAJ
!
!---------------   COMPONENT ON y-AXIS
!
  Z=F1+F2-1./(RMAJ*RMAJ)
  if ( Z<=0. ) then
     IERR=-2
     return
  end if
  RMIN=1./sqrt(Z)
  B=RMIN
  IREVER=0
  if ( RMAJ<RMIN) then
     IREVER=1
     TT=RMAJ
     RMAJ=RMIN
     RMIN=TT
  end if
!
!---------------    ANGLE
!
!  1)    x/HORIZONTAL
!    SIGN - BECAUSE IT IS THE REVERSE THAT WE WANT (FROM NORMAL TO ELLIPSE)
!
  G1=-F3/(F1-F2+1./(A*A)-1./(B*B))
  ARGUM=G1
  ANG=ATAN(ARGUM)*FACT
!
!  2)  CHECK IF x IS MAJOR AXIS
!
  if ( IREVER==0 ) GO TO 10
  if ( ANG>=0. ) GO TO 11
  ANG=ANG+90.
  GO TO 10
11 ANG=ANG-90.
!
!  3)  ANGLE MAJOR AXIS/VERTICAL, UNCLOCKWISE
!
10 ANG=ANG+90.
  
  SRMAJ=RMAJ
  SRMIN=RMIN
  SANG =ANG
  SG   =SIGM
  
  return
END SUBROUTINE FITE
!***********************************************************************C
SUBROUTINE LEAST (OBS,M,N,NT,F,SG,ier)
!	
!	The least-squares fitting routine used in FITELL.
!
  parameter (MAXPNT=500)
  implicit real*8	 (A-H,O-Z)
  dimension OBS(MAXPNT,6), F(5), C(10,11),S(10)
  
  ier = 0
  do  J=1,10
     S(J)=0.D0
     do  I=1,11
        C(J,I)=0.D0
     end do
  end do
  SIGMA=0.
  L=N-1
  
  do  I=1,L
     do  J=1,N
        T=0.0D0
        do  K=1,NT
           Z1=dble( OBS(K,I) )
           Z2=dble( OBS(K,J) )
           T=T+Z1*Z2
        end do
        C(I,J) = T
     end do
  end do
!-----------------
!   SOLVING BY GAUSSIAN ELIMINATION
!-----------------
  call LNSYS(C,S,L,ier)
!-----------------
!  SET UP COEFFICIENTS
!-----------------
  do  I=1,L
     F(I)=S(I)
  end do
  do  I=1,NT
     ZZ=dble( OBS(I,N) )
     do  J=1,L
        ZZ=ZZ-S(J)*dble( OBS(I,J) )
     end do
     if ( ZZ<=0. ) ZZ=-ZZ
     SIGMA=SIGMA+ZZ
  end do
  SIGMA=(SIGMA/real(NT))*sqrt(3.141593D0/2.D0)
  SG=SIGMA
  
  return

END SUBROUTINE LEAST

!***********************************************************C
!       SOLVE LINEAR SYSTEM VIA GAUSSIAN ELIMINATION        C
!          ORDER UP TO 10                                   C
!             ( 10.01.1979 K. BANSE )                       C
!                              (SACLAY - HPCCD - 1982/84)   C
!		REVISED BY M.HAMABE			        C
!***********************************************************C
SUBROUTINE LNSYS (A,X,N,ier)
!
!	The routine used in the FITELL.
!
  implicit  real*8 (A-H,O-Z)
  dimension A(10,11), X(10)
!
  ier = 0
  do  K=1,N-1
     SAVE=0.D0			
!						! GET LARGEST PIVOTAL ELEMENT
     do  J=K,N
        RMAX=ABS(A(J,K))
        if ( SAVE<RMAX ) then
           SAVE=RMAX
           MXROW=J
        end if
     end do
     do  J=K,N+1			
!						!  EXCHANGE ROWS
        SAVE=A(K,J)
        A(K,J)=A(MXROW,J)
        A(MXROW,J)=SAVE
     end do
     if ( ABS(A(K,K))<10.D-12 ) then	
!						! TEST IF ARRAY SINGULAR
        do  I=1,N
           X(I)=0.0
        end do
        ier = -1
        return
     end if
     do  I=K+1,N				
!						! SUBSTRACT ROW FROM LOWER ROWS
        SAVE=A(I,K)/A(K,K)
        A(I,K)=0.D0
        do  J=K+1,N+1
           A(I,J)=A(I,J)-SAVE*A(K,J)
        end do
     end do
  end do
!
!       CALCULATE VECTOR X
!
  X(N)=A(N,N+1)/A(N,N)
  do  K=2,N
     I=N+1-K
     SAVE=A(I,N+1)
     do  J=I+1,N
        SAVE=SAVE-A(I,J)*X(J)
     end do
     X(I)=SAVE/A(I,I)
  end do
  
  return
END SUBROUTINE LNSYS
!***********************************************************************C
!         GENERALIZED LEAST-SQUARE FOR UP TO 10 PARAMETERS              C
!                     (Jean SOUVIRON - DAO Victoria - 1985)             C
!-----------------------------------------------------------------------C
!                                                                       C
!     INPUT :                                                           C
!           NARAY = NUMBER OF DIFFERENT PARAMETERS+1                    C
!            NDIM = real MAX dimension IN THE MAIN PROGRAM              C
!          NTOTAL = TOTAL NUMBER OF POINTS                              C
!          TABOBS = ARRAY OF PARAMETERS FOR EACH POINT                  C
!                   (LAST VECTOR HAS TO BE THE DATA TO BE FITTED)       C
!     OUTPUT :                                                          C
!          FACTOR = ARRAY OF COEFFICIENTS                               C
!                       (dimension MUST BE BUILD IN IN MAIN PROGRAM)    C
!          SIGMA                                                        C
!                                                                       C
!***********************************************************************C
SUBROUTINE LEAST2 (TABOBS,NDIM,NARAY,NTOTAL,FACTOR,SIGMA)

  real(4) :: TABOBS(NDIM,NARAY),FACTOR(1)
  real(8) :: COEFF(10,11),SOLU(10)

  do J=1,10
     SOLU(J)=0.D0
     do I=1,11
        COEFF(J,I)=0.D0
     end do
  end do
  SIGMA=0.
  Nreal=NARAY-1
  do I=1,Nreal
     do J=1,NARAY
        T=0.0
        do K=1,NTOTAL
           Z1=TABOBS(K,I)
           Z2=TABOBS(K,J)
           T=T+Z1*Z2
        end do
        COEFF(I,J)=dble(T)
     end do
  end do
!-----------------
!   SOLVING BY GAUSSIAN ELIMINATION
!-----------------
  call LNSYS(COEFF,SOLU,Nreal,ier)
!-----------------
!  SET UP COEFFICIENTS
!-----------------
  do I=1,Nreal
     FACTOR(I)=SOLU(I)
  end do
  do I=1,NTOTAL
     ZZ=TABOBS(I,NARAY)
     do J=1,Nreal
        ZZ=ZZ-FACTOR(J)*TABOBS(I,J)
     end do
     if (ZZ<=0.)ZZ=-ZZ
     SIGMA=SIGMA+ZZ
  end do
  SIGMA=(SIGMA/real(NTOTAL))*sqrt(3.14159/2.)

  return

END SUBROUTINE LEAST2
