PROGRAM  vista_profile
!     
!     MODIFIED 1989-07-23 BY HAMABE
!     Updated: M.Hamabe 27/09/1990 (imfort version)
!     Updated: M.Hamabe 01/03/1991
!     Updated: M.Hamabe 19/03/1991  (SPIRAL.SIZE)
!     Updated: M.Hamabe  1991-09-10  (opninp,matrd)
!     Updated: M.Hamabe  1991-12-27  (suppress message)
!     Updated: M.Hamabe  1992-09-11  (display format changed for large radii)
!     Updated: M.Hamabe  1992-11-18  (overwrite parameters)
!     Updated: M.Hamabe  1995-06-29  (SECPPIX)
!     Updated: M.Hamabe  1996-01-12  (print center)
!     Updated: M.Hamabe  1996-08-28  (support FITS)
!     Updated: M.Hamabe  1996-09-30  (bug-fix)
!     Updated: M.Hamabe  1999-07-23  (omit IMFORT)
!     Updated: M.Hamabe  1999-08-16  (increase NPROFI)
!     Updated: M.Hamabe  1999-08-19  (fixed axial-ratio; CDELT1 in degree)
!     Updated: M.Hamabe  1999-09-05  (make a new subroutine for param.input)
!     Updated: M.Hamabe  2000-07-24  (bugfix; error recovery)
!     Updated: M.Hamabe  2002-10-21  (DLINE=blank)
!     Updated: M.Hamabe  2007-10-15  (for Fortran90)
!             Caution: Go to 6000/7000 are incomplete
      
  parameter (maxdim=1024)
  
  character	 image*80,title*18,band*2
  character	 tabfile*30, tabl*30, extend*30
  integer	 im, irc
  dimension  g(maxdim*maxdim)
  logical    INTER, CENTER, LOGSC, ERRLOG, NOTFRE, ANGFRE, RATFIX
  
  NLP =16
  LOUT=21
  
  call opninp (1,im,0,image,title,iext,jext,irc)
  if ( irc/=0 ) go to 999
  
  call clargc (2, tabl, irc)
  if ( irc.ne.0 ) then
     print '(''Enter output profile-table name: '',$)'
     read *, tabl
  end if
  tabfile=extend(tabl,'pr')
  open (LOUT, file=tabfile,access='SEQUENTIAL')
      
!     -----  color band
  call GETKEYS(im,'BAND    ',band,irc)
!     call GETKEYS(im,'FILTNAM1',filtname,irc)
  if ( irc.ne.0 ) then
     print '(''Enter color band (A2)     ===> '',$)'
     read '(a)', band
     irc=0
  end if
  print '(''     Band = '',a2)', band
      
!     -----  pixel scale
  call GETKEYE(im,'CDELT1',secppix,irc)
  if ( irc.ne.0 ) call GETKEYE(im,'SECPPIX ',secppix,irc)
  if ( irc.ne.0 ) then
     print '(''Enter pixel scale (arcsec/pixel)     ===> '',$)'
     read *, secppix
     irc=0
  end if
  secppix=abs(secppix)
  if ( secppix.lt.10/3600.0 ) secppix=secppix*3600.0
  if ( secppix.lt.0.001 .or. secppix.gt.10.0 ) irc=1
  print '(''     Pixel size = '',f7.3,'' arcsec'')', secppix
      
!     ----- Sky surface brightness
  call GETKEYE(im, 'UNIT-MAG', sky, irc)
  if ( irc.ne.0 ) then
     print '(''Enter surface brightness unit (sky)  ===> '',$)'
     read *, sky
     irc=0
  end if
  print '(''     Sky = '',f7.3,'' mag arcsec**(-2)'')', sky
  
  call GETKEYJ(im,'BLANK',iblank,irc)
  if ( irc.ne.0 ) then
     call GETKEYJ(im,'IGNRVAL',iblank,irc)
     if ( irc.ne.0 ) then
        irc=0
        blank=-32768.0
        print '(''BLANK is not defined'')'
     else
        print '(''IGNRVAL is assumed to be BLANK'')'
        blank=iblank
     end if
  else
     blank=iblank
  end if
  print '('' ---> BLANK is assumed to '',f15.7)', blank
  
  call paramin(XCAX,YCAX,FINC,POSANG,ANGSGN,ANGAX,boa_0, &
       &       NSTEP,MPA,NSINC,NAKA,NITER,NROUGH,LOM1,LOM2,NLP, &
       &       INTER,CENTER,LOGSC,ERRLOG,NOTFRE,ANGFRE,RATFIX)
      
  PRINT *
      
!     *** READ MATRIX ***
  CALL MATRD(im,G,IEXT,JEXT,irc)
  
  CALL SUBPROF(title,G,IEXT,JEXT,secppix,sky,band,NLP,LOUT, &
       &       XCAX,YCAX,FINC,POSANG,ANGSGN,ANGAX,boa_0,blank, &
       &       NSTEP,MPA,NSINC,NAKA,NITER,NROUGH,LOM1,LOM2, &
       &       INTER,CENTER,LOGSC,ERRLOG,NOTFRE,ANGFRE,RATFIX)
  
  stop
  
999 call PUTEMSG(irc)
  stop
  
end PROGRAM vista_profile

!-------------------------------------------------------------------------

subroutine paramin(XCAX,YCAX,Finc,POSANG,ANGSGN,ANGAX,boa_0, &
     &             NSTEP,MPA,NSINC,NAKA,NITER,NROUGH,LOM1,LOM2,NLP, &
     &             INTER,CENTER,LOGSC,ERRLOG,NOTFRE,ANGFRE,RATFIX)
  
  character  NR, reverse*3, meanpa*3, cnum*10
  character  solcen*3, fixpa*3, fixab*3, iiter*3, maklog*3
  logical    INTER, CENTER, LOGSC, ERRLOG, NOTFRE, ANGFRE, RATFIX

!     ### Initialize parameters

  Finc = 1.0                        ! Iteration increment
  
  posang=0.0
  NR='N'
  call clargc (3, reverse, irc)
  if ( irc.ne.0 ) then
     print '(''Image is reverse? ( y/<n> ) ===> '',$)'
     read '(A)', reverse
     irc=0
  end if
  if ( reverse(1:1).eq.'y' ) then
     NR='R'
  end if
  if ( NR.EQ.'N' ) then
     PRINT *, 'Image is normal   (no mirror reflection)'
     ANGSGN = -1.0                  ! No mirror reflections
  else if ( NR.EQ.'R' ) then
     PRINT *, 'Image is reversed (mirror reflection)'
     ANGSGN =  1.0
  end if
  
  call clargr (4, POSANG, irc)
  if ( irc.ne.0 ) then
     print '(''Enter Position angle: '',$)'
     read *, POSANG
     irc=0
  end if
  if ( POSANG.LT.0.0 ) then
     POSANG = -POSANG
     ANGSGN = 1.0
  end if
      
!     ----- Center of galaxy
  call clargr (5, xcax, irc)
  if ( irc.ne.0 ) then
     print '(''Enter x-coordinate of the center of galaxy: '',$)'
     read *, xcax
     irc=0
  end if
  call clargr (6, ycax, irc)
  if ( irc.ne.0 ) then
     print '(''Enter y-coordinate of the center of galaxy: '',$)'
     read *, ycax
     irc=0
  end if
      
!     ----- Position angle of the major axis of the object
  call clargr (7, dum, irc)
  if ( irc.ne.0 ) then
         print '(''Enter Position angle of the object (top ->left): '',$)'
         read *, dum
         irc=0
      end if
      ANGAX = 180.0 - DUM ! <= angle between i-axis and major-axis (clockwise)
!					     ANGAX = 270.0-DUM

!     -----  keyword is Mean Position Angle
      MPA = 3              ! DEFAULT radius+1 in which mean PA used
      NOTFRE = .FALSE.
      call clargc (8, meanpa, irc)
!     print '(
!     +    ''Use mean position angle for center ( y/<n> ) ===> '',$)'
!     read '(A)', YN
      if ( meanpa.eq.'yes' ) then
         print '(''How many contours do you use to calculate the '', &
              &    ''mean position angle? <mpa=3> ===> '',$)'
         read '(a)', cnum
         if ( cnum.ne.' ' )  read (cnum,'(BN,i10)') mpa
         NOTFRE = .TRUE.
      end if
      irc=0
      
!     -----  keyword RADIUS=
      call clargi (9, NSTEP, irc)
      if ( irc.ne.0 ) then
         print '(''Enter Maximum radius (r): '',$)'
         read *, NSTEP
         irc=0
      end if

!     -----  keyword M=
      call clargi (10, NSINC, irc)
      if ( irc.ne.0 ) then
         print '(''Enter Number of SINC interpolation (m): '',$)'
         read *, NSINC
         irc=0
      end if

!     -----  keyword is LOGSC
      LOGSC  = .FALSE.
!     print '(''Would you like to use log-interval ? ( y/<n> ) ===> '',$)'
!     read '(A)', YN
!     if ( yn.eq.'y' ) then
!         LOGSC = .TRUE.
!         print '(''     delta log(r) ===> '',$)')
!         read *, DLTLGR
!         FCTRAD= 10.0**(DLTLGR)
!     end if

! -----  keyword N=
      if ( LOGSC ) then
         NAKA   = 0
         print '(''Number of contours in 1-pix. interval (n) ===> '',$)'
         read '(A10)', CNUM
         if ( CNUM.EQ.' ' ) CNUM='         0'
         READ (CNUM,'(BN,I10)') NAKA
      else
         NAKA=NSTEP
      end if
      
      !     -----  keyword ITER=
      NITER = 2                 ! Number of iterations
      NROUGH = NITER            ! Number with bilinear interpolation
      call clargi (11, NROUGH, irc)
      if ( irc.ne.0 ) then
         print '(''Enter Number of Fast BILINEAR interpolation= '',$)'
         read *, NROUGH
         irc=0
      end if
      call clargi (12, NDUM, irc)
      if ( irc.ne.0 ) then
         print '(''Enter Number of Slower SINC interpolation= '',$)'
         read *, NDUM
         irc=0
      end if
      NITER  = NROUGH + NDUM

!     -----  keyword CENTER
      CENTER = .FALSE.
      call clargc (13, solcen, irc)
!     print '(''Solve the center ?       ( y/<n> ) ===> '',$)'
!     read '(A)', YN
      if ( solcen.eq.'yes' ) center=.true.

!     -----  keyword ANGFRE
      ANGFRE = .TRUE.
      call clargc (14, fixpa, irc)
      if ( irc.ne.0 ) then
         print '(''Fix the position angle ? ( y/<n> ) ===> '',$)'
         read '(A)', fixpa
         irc=0
      end if
      if ( fixpa(1:1).eq.'y' ) ANGFRE=.false.

!     ===== for fiexd b/a (added 19 Aug., 1999)
      RATFIX = .FALSE.
      call clargc (15, fixab, irc)
      if ( irc.ne.0 ) then
         print '(''Fix the axial ratio ? ( y/<n> ) ===> '',$)'
         read '(A)', fixab
         irc=0
      end if
      if ( fixab(1:1).eq.'y' ) then
         RATFIX=.TRUE.
         call clargr (16,boa_0,irc)
         if ( irc.ne.0 ) then
            print '(''axial ratio ?  = '',$)'
            read *, boa_0
            irc=0
         end if
      end if

!     -----  Definition of the omitting radius range 
      call clargi (17, LOM1, irc)
      if ( irc.ne.0 ) LOM1 = 0

      call clargi (18, LOM2, irc)
      if ( irc.ne.0 ) LOM2 = 0

!     -----  keyword is NOINT
      INTER = .TRUE.              ! use interaction to decide when to stop
      call clargc (19, iiter, irc)
      if ( irc.ne.0 ) then
         print '(''Interactive iteration ? ( <y>/n ) ===> '',$)'
         read '(A)', iiter
         irc=0
      end if
      if ( iiter(1:1).eq.'n' ) then
         INTER = .FALSE.
      end if

!     -----  keyword is LOG
      ERRLOG = .FALSE.
      call clargc (20, maklog, irc)
      if ( irc.ne.0 ) then
         print '(''Make the log of iteration? ( <y>/n ) ===> '',$)'
         read '(A)', maklog
         irc=0
      end if
      if ( maklog(1:1).eq.'y' ) then
         errlog = .true.
         open (NLP,  file='profile.log',access='SEQUENTIAL')
      end if

!     -------------------------------------------------------------

!     Make sure that the limits on contours
!         ( R-NSTEP, N-NAKA, and M-NSINC )
!     All exist and have legitimate values.

      if ( LOGSC ) then
3        if ( NSTEP.le.0 ) then
            print '(''Enter maximum radius R ===> '',$)'
            read *, NSTEP
            go to 3
         end if
         if ( NAKA.eq.0 ) NAKA  = NSINC
      else
4        if ( NSTEP.le.0 .and. NAKA.le.0 ) then
         if ( NSINC.gt.0 ) then
            NSTEP = NSINC
            NAKA = NSINC
         else
            print '(''Number n of profile steps in pixels: '',$)'
            read *, NSTEP
            NAKA  = NSTEP
            print *, 'NAKA',NAKA,' NSTEP',NSTEP
            go to 4
         end if
      else if ( NSTEP.le.0 ) then
         NSTEP = NAKA
      else
         NAKA  = NSTEP
      end if
   end if

!@@@  
   if ( NSINC.gt.NAKA ) NSINC=NAKA
   
   return
 end subroutine paramin
